# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


class PolyaxonK8SError(Exception):
    """Exception class to raise in case of a Kubernetes issue."""
    pass
